

library(copula)

library(GFGM.copula)
# Loading required package: cmprsk
# Loading required package: survival
# Loading required package: compound.Cox
# Loading required package: numDeriv
# Loading required package: joint.Cox

# BurrIII -------------------------------------------------------

dBurrIII=function(x,alpha,gamma,eta=0){
  x=x-eta
  # logd=log(alpha)+log(gamma)-(alpha+1)*log(x)-(alpha+1)log(1+1/x^gamma)
  d=alpha*gamma/(x^(gamma+1)*(1+1/x^gamma)^(alpha+1))
  return(d)
}

pBurrIII=function(x,alpha,gamma,eta=0){
  x=x-eta
  p=1/(1+1/x^gamma)^alpha
  return(p)
}

mBurrIII=function(alpha,gamma){
  m=alpha*beta(alpha+1/gamma,1-1/gamma)
  return(m)
}

# CvM -----------------------------------------------------------

cvm.GFGM.BurrIII=function (t.event, event1, event2, Alpha, Beta, Gamma, g1, g2, 
          p, q, theta, eta = 0, Sdist.plot = TRUE,ymax=1) {
  event = event1 + 2 * event2
  t.data = data.frame(t.event, event)
  t1.event = unique(sort(t.data$t.event[event == 1]))
  t2.event = unique(sort(t.data$t.event[event == 2]))
  res.cuminc = cuminc(t.event, event)
  tmin = min(t.event)
  tmax = max(t.event)
  Sdist.BIII1 = Sdist.GFGM.BurrIII(t1.event, Alpha, Beta, 
                                   Gamma, p, q, theta, eta = eta)[, 2]
  Sdist.BIII2 = Sdist.GFGM.BurrIII(t2.event, Alpha, Beta, 
                                   Gamma, p, q, theta, eta = eta)[, 3]
  Sdist.spline1 = Sdist.GFGM.spline(t1.event, g1, g2, tmin, 
                                    tmax, p, q, theta)[, 2]
  Sdist.spline2 = Sdist.GFGM.spline(t2.event, g1, g2, tmin, 
                                    tmax, p, q, theta)[, 3]
  Sdist.non1 = as.vector(timepoints(res.cuminc, t1.event)$est[1, 
                                                              ])
  Sdist.non2 = as.vector(timepoints(res.cuminc, t2.event)$est[2, 
                                                              ])
  S.overall = sum((Sdist.BIII1 - Sdist.non1)^2) + sum((Sdist.BIII2 - 
                                                         Sdist.non2)^2)
  S.GFGM = sum((Sdist.spline1 - Sdist.non1)^2) + sum((Sdist.spline2 - 
                                                        Sdist.non2)^2)
  if (Sdist.plot) {
    t.grid = seq(tmin, tmax, length.out = 500)
    Sdist.BurrIII = Sdist.GFGM.BurrIII(t.grid, Alpha, Beta, 
                                       Gamma, p, q, theta, eta)
    Sdist.spline = Sdist.GFGM.spline(t.grid, g1, g2, tmin, 
                                     tmax, p, q, theta)
    plot(res.cuminc$`1 1`$time, res.cuminc$`1 1`$est, xlab = "t", 
         ylim = c(0, ymax), xlim = c(tmin, tmax), ylab = "Sub-distribution", 
         type = "s")
    lines(Sdist.BurrIII[, 1], Sdist.BurrIII[, 2], type = "l", 
          col = "blue")
    lines(Sdist.spline[, 1], Sdist.spline[, 2], type = "l", 
          col = "red")
    lines(res.cuminc$`1 2`$time, res.cuminc$`1 2`$est, 
         type = "s",lty=2)
    lines(Sdist.BurrIII[, 1], Sdist.BurrIII[, 3], type = "l", 
          col = "blue", lty = 2)
    lines(Sdist.spline[, 1], Sdist.spline[, 3], type = "l", 
          col = "red", lty = 2)
    legend("topleft", c("para 1", "para 2", 
                        "semi-para 1", "semi-para 2", 
                        "non-para 1", "non-para 2"), 
           lty = rep(1:2,times=3), col = rep(c("blue", "red", "black"),each=2))
  }
  return(list(S.overall = S.overall, S.GFGM = S.GFGM))
}

# mle.sim -------------------------------------------------------

data.sim <- function(n, Alpha, Beta, Gamma,w,
                     p=3,q=2,theta=0.7) {
  XY=GFGM.BurrIII(n, p, q, theta, Alpha, Beta, Gamma)
  X=XY[,1]
  Y=XY[,2]
  cen=runif(n,0,w)
  XYC=cbind(XY,cen)
  
  t.event = apply(XYC, 1, min)
  event1  = as.integer(t.event==X)
  event2  = as.integer(t.event==Y)
  event3  = 1-event1-event2
  
  Time=apply(XYC, 1, which.min)
  Time=as.factor(Time)
  levels(Time) = c("Confirmed","Unconfirmed","Censoring")
  data1   = data.frame(t.event,Time,event1,event2,event3)
  return(data1)
}

mle.sim <- function(n, Alpha, Beta, Gamma,w,
                    D=5000,p=3,q=2,theta=0.7,
                    id=NULL) {
  data1=data.sim(n, Alpha, Beta, Gamma,w, p, q, theta)
  t.event=data1$t.event
  event1=data1$event1
  event2=data1$event2
  
  res.BurrIII=MLE.GFGM.BurrIII(t.event,event1,event2,D,p,q,theta)
  res.mat=as.data.frame(res.BurrIII[4:8])
  Real=c(Alpha,Beta,Gamma,mBurrIII(Alpha,Gamma),mBurrIII(Beta,Gamma))
  Coverage=(res.mat["CI.lower",]<=Real) & (Real<=res.mat["CI.upper",])
  res.mat.ext=rbind(res.mat,Real=Real,Coverage=Coverage,
                    Iteration=res.BurrIII$Iteration,
                    Randomization=res.BurrIII$Randomization)
  res.mat.ext=res.mat.ext[c(1,2,6,7,8),]
  if(! is.null(id)){
    id<<-id+1
    cat(id," ")
  }
  return(as.matrix(res.mat.ext))
}

# Copula Sim ----------------------------------------------------

GFGM.exp=function (n, p, q, theta, Lambda1, Lambda2) {
  U = rep(0, n)
  V = rep(0, n)
  for (i in 1:n) {
    v = runif(1)
    a = runif(1)
    K = (1 - v^p)^(q - 1) * (1 - (1 + p * q) * v^p)
    f = function(u) {
      u + theta * K * u * (1 - u^p)^q - a
    }
    u = uniroot(f, c(0, 1), tol = 1e-09)
    U[i] = u$root
    V[i] = v
  }
  X = qexp(U,Lambda1)
  Y = qexp(V,Lambda2)
  return(cbind(X, Y))
}

Clayton.BurrIII=function(n, theta, Alpha, Beta, Gamma){
  Clayton.cop <- claytonCopula(param=theta, dim=2)
  UV=rCopula(n, Clayton.cop)
  U=UV[,1]
  V=UV[,2]
  X = (U^(-1/Alpha) - 1)^(-1/Gamma)
  Y = (V^(-1/Beta) - 1)^(-1/Gamma)
  return(cbind(X, Y))
}

Clayton.exp=function(n, theta, Lambda1, Lambda2){
  Clayton.cop <- claytonCopula(param=theta, dim=2)
  UV=rCopula(n, Clayton.cop)
  U=UV[,1]
  V=UV[,2]
  X = qexp(U,Lambda1)
  Y = qexp(V,Lambda2)
  return(cbind(X, Y))
}

Indep.BurrIII=function (n, Alpha, Beta, Gamma){
  U = runif(n)
  V = runif(n)
  X = (U^(-1/Alpha) - 1)^(-1/Gamma)
  Y = (V^(-1/Beta) - 1)^(-1/Gamma)
  return(cbind(X, Y))
}

Indep.exp=function (n, Lambda1, Lambda2){
  X = rexp(n,Lambda1)
  Y = rexp(n,Lambda2)
  return(cbind(X, Y))
}

# gof.sim -------------------------------------------------------

gof.sim.BurrIII <- function(n, Alpha, Beta, Gamma,w,
                            D=5000,p=3,q=2,theta=0.7,
                            id=NULL) {
  data1=data.sim(n, Alpha, Beta, Gamma,w, p, q, theta)
  t.event=data1$t.event
  event1=data1$event1
  event2=data1$event2
  
  res.BurrIII=MLE.GFGM.BurrIII(t.event,event1,event2,D,p,q,theta)
  
  Alpha.h = res.BurrIII$Alpha[1]
  Beta.h = res.BurrIII$Beta[1]
  Gamma.h = res.BurrIII$Gamma[1]
  
  res.spline = MLE.GFGM.spline(t.event,event1,event2,p,q,theta,h.plot=F)
  g1 = res.spline$g1
  g2 = res.spline$g2
  
  res=CvM.GFGM.BurrIII(t.event,event1,event2,Alpha.h,Beta.h,Gamma.h,g1,g2,p,q,theta,Sdist.plot =F)
  res=unlist(res)
  if(! is.null(id)){
    id<<-id+1
    cat(id," ")
  }
  return(res)
}

boot.gof.sim.BurrIII <- function(n=200, Alpha=1.5, Beta=1, Gamma=1,w=6,
                            D=5000,p=3,q=2,theta=0.7,
                            id=NULL,B=500,alpha=0.05,.progress="text") {
  data1=data.sim(n, Alpha, Beta, Gamma,w, p, q, theta)
  t.event=data1$t.event
  event1=data1$event1
  event2=data1$event2
  event3=data1$event3
  
  res.BurrIII=MLE.GFGM.BurrIII(t.event,event1,event2,D,p,q,theta)
  
  Alpha.h = res.BurrIII$Alpha[1]
  Beta.h = res.BurrIII$Beta[1]
  Gamma.h = res.BurrIII$Gamma[1]
  
  res.spline = MLE.GFGM.spline(t.event,event1,event2,p,q,theta,h.plot=F)
  g1 = res.spline$g1
  g2 = res.spline$g2
  
  stat0=CvM.GFGM.BurrIII(t.event,event1,event2,Alpha.h,Beta.h,Gamma.h,g1,g2,p,q,theta,Sdist.plot =F)
  stat0=unlist(stat0)
  
  w.boot=max(t.event[event3==1])
  boot.sim=plyr::raply(B,gof.sim.BurrIII(n, Alpha.h, Beta.h, Gamma.h,w.boot),.progress=.progress)
  stat.boot=colMeans(boot.sim)
  q.boot=apply(boot.sim,2,quantile,probs=1-alpha)
  reject=stat0>q.boot
  res=c(stat0[1],stat.boot[1],reject[1],stat0[2],stat.boot[2],reject[2])
  names(res)=c("overall.stat","overall.boot.stat","overall.reject",
               "copula.stat","copula.boot.stat","copula.reject")
  return(res)
}

data.sim.GFGM.exp <- function(n, Lambda1=0.5, Lambda2=1,w,
                     p=3,q=2,theta=0.7) {
  XY=GFGM.exp(n, p, q, theta, Lambda1, Lambda2)
  X=XY[,1]
  Y=XY[,2]
  cen=runif(n,0,w)
  XYC=cbind(XY,cen)
  
  t.event = apply(XYC, 1, min)
  event1  = as.integer(t.event==X)
  event2  = as.integer(t.event==Y)
  event3  = 1-event1-event2
  
  Time=apply(XYC, 1, which.min)
  Time=as.factor(Time)
  levels(Time) = c("Confirmed","Unconfirmed","Censoring")
  data1   = data.frame(t.event,Time,event1,event2,event3)
  return(data1)
}

boot.gof.sim.GFGM.exp <- function(n=200, Lambda1=0.5, Lambda2=1,w=6,
                            D=5000,p=3,q=2,theta=0.7,
                            id=NULL,B=500,alpha=0.05,.progress="text") {
  data1=data.sim.GFGM.exp(n, Lambda1, Lambda2,w, p, q, theta)
  t.event=data1$t.event
  event1=data1$event1
  event2=data1$event2
  event3=data1$event3
  
  res.BurrIII=MLE.GFGM.BurrIII(t.event,event1,event2,D,p,q,theta)
  
  Alpha.h = res.BurrIII$Alpha[1]
  Beta.h = res.BurrIII$Beta[1]
  Gamma.h = res.BurrIII$Gamma[1]
  
  res.spline = MLE.GFGM.spline(t.event,event1,event2,p,q,theta,h.plot=F)
  g1 = res.spline$g1
  g2 = res.spline$g2
  
  stat0=CvM.GFGM.BurrIII(t.event,event1,event2,Alpha.h,Beta.h,Gamma.h,g1,g2,p,q,theta,Sdist.plot =F)
  stat0=unlist(stat0)
  
  w.boot=max(t.event[event3==1])
  boot.sim=plyr::raply(B,gof.sim.BurrIII(n, Alpha.h, Beta.h, Gamma.h,w.boot),.progress=.progress)
  stat.boot=colMeans(boot.sim)
  q.boot=apply(boot.sim,2,quantile,probs=1-alpha)
  reject=stat0>q.boot
  res=c(stat0[1],stat.boot[1],reject[1],stat0[2],stat.boot[2],reject[2])
  names(res)=c("overall.stat","overall.boot.stat","overall.reject",
               "copula.stat","copula.boot.stat","copula.reject")
  return(res)
}

data.sim.Clayton.BurrIII <- function(n, Alpha, Beta, Gamma,w,
                     theta=0.7) {
  XY=Clayton.BurrIII(n, theta, Alpha, Beta, Gamma)
  X=XY[,1]
  Y=XY[,2]
  cen=runif(n,0,w)
  XYC=cbind(XY,cen)
  
  t.event = apply(XYC, 1, min)
  event1  = as.integer(t.event==X)
  event2  = as.integer(t.event==Y)
  event3  = 1-event1-event2
  
  Time=apply(XYC, 1, which.min)
  Time=as.factor(Time)
  levels(Time) = c("Confirmed","Unconfirmed","Censoring")
  data1   = data.frame(t.event,Time,event1,event2,event3)
  return(data1)
}

boot.gof.Clayton.BurrIII <- function(n=200, Alpha=1.5, Beta=1, Gamma=1,w=6,
                            D=5000,p=3,q=2,theta=0.7,
                            id=NULL,B=500,alpha=0.05,.progress="text") {
  data1=data.sim.Clayton.BurrIII(n,  Alpha, Beta, Gamma,w, theta)
  t.event=data1$t.event
  event1=data1$event1
  event2=data1$event2
  event3=data1$event3
  
  res.BurrIII=MLE.GFGM.BurrIII(t.event,event1,event2,D,p,q,theta)
  
  Alpha.h = res.BurrIII$Alpha[1]
  Beta.h = res.BurrIII$Beta[1]
  Gamma.h = res.BurrIII$Gamma[1]
  
  res.spline = MLE.GFGM.spline(t.event,event1,event2,p,q,theta,h.plot=F)
  g1 = res.spline$g1
  g2 = res.spline$g2
  
  stat0=CvM.GFGM.BurrIII(t.event,event1,event2,Alpha.h,Beta.h,Gamma.h,g1,g2,p,q,theta,Sdist.plot =F)
  stat0=unlist(stat0)
  
  w.boot=max(t.event[event3==1])
  boot.sim=plyr::raply(B,gof.sim.BurrIII(n, Alpha.h, Beta.h, Gamma.h,w.boot),.progress=.progress)
  stat.boot=colMeans(boot.sim)
  q.boot=apply(boot.sim,2,quantile,probs=1-alpha)
  reject=stat0>q.boot
  res=c(stat0[1],stat.boot[1],reject[1],stat0[2],stat.boot[2],reject[2])
  names(res)=c("overall.stat","overall.boot.stat","overall.reject",
               "copula.stat","copula.boot.stat","copula.reject")
  return(res)
}

data.sim.Clayton.exp <- function(n, Lambda1=0.5, Lambda2=1,w,
                     theta=0.7) {
  XY=Clayton.exp(n, theta, Lambda1, Lambda2)
  X=XY[,1]
  Y=XY[,2]
  cen=runif(n,0,w)
  XYC=cbind(XY,cen)
  
  t.event = apply(XYC, 1, min)
  event1  = as.integer(t.event==X)
  event2  = as.integer(t.event==Y)
  event3  = 1-event1-event2
  
  Time=apply(XYC, 1, which.min)
  Time=as.factor(Time)
  levels(Time) = c("Confirmed","Unconfirmed","Censoring")
  data1   = data.frame(t.event,Time,event1,event2,event3)
  return(data1)
}

boot.gof.Clayton.exp <- function(n=200, Lambda1=0.5, Lambda2=1,w=6,
                            D=5000,p=3,q=2,theta=0.7,
                            id=NULL,B=500,alpha=0.05,.progress="text") {
  data1=data.sim.Clayton.exp(n,  Lambda1, Lambda2,w, theta)
  t.event=data1$t.event
  event1=data1$event1
  event2=data1$event2
  event3=data1$event3
  
  res.BurrIII=MLE.GFGM.BurrIII(t.event,event1,event2,D,p,q,theta)
  
  Alpha.h = res.BurrIII$Alpha[1]
  Beta.h = res.BurrIII$Beta[1]
  Gamma.h = res.BurrIII$Gamma[1]
  
  res.spline = MLE.GFGM.spline(t.event,event1,event2,p,q,theta,h.plot=F)
  g1 = res.spline$g1
  g2 = res.spline$g2
  
  stat0=CvM.GFGM.BurrIII(t.event,event1,event2,Alpha.h,Beta.h,Gamma.h,g1,g2,p,q,theta,Sdist.plot =F)
  stat0=unlist(stat0)
  
  w.boot=max(t.event[event3==1])
  boot.sim=plyr::raply(B,gof.sim.BurrIII(n, Alpha.h, Beta.h, Gamma.h,w.boot),.progress=.progress)
  stat.boot=colMeans(boot.sim)
  q.boot=apply(boot.sim,2,quantile,probs=1-alpha)
  reject=stat0>q.boot
  res=c(stat0[1],stat.boot[1],reject[1],stat0[2],stat.boot[2],reject[2])
  names(res)=c("overall.stat","overall.boot.stat","overall.reject",
               "copula.stat","copula.boot.stat","copula.reject")
  return(res)
}

data.sim.Indep.BurrIII <- function(n, Alpha, Beta, Gamma,w) {
  XY=Indep.BurrIII(n, Alpha, Beta, Gamma)
  X=XY[,1]
  Y=XY[,2]
  cen=runif(n,0,w)
  XYC=cbind(XY,cen)
  
  t.event = apply(XYC, 1, min)
  event1  = as.integer(t.event==X)
  event2  = as.integer(t.event==Y)
  event3  = 1-event1-event2
  
  Time=apply(XYC, 1, which.min)
  Time=as.factor(Time)
  levels(Time) = c("Confirmed","Unconfirmed","Censoring")
  data1   = data.frame(t.event,Time,event1,event2,event3)
  return(data1)
}

boot.gof.Indep.BurrIII <- function(n=200, Alpha=1.5, Beta=1, Gamma=1,w=6,
                            D=5000,p=3,q=2,theta=0.7,
                            id=NULL,B=500,alpha=0.05,.progress="text") {
  data1=data.sim.Indep.BurrIII(n,  Alpha, Beta, Gamma,w)
  t.event=data1$t.event
  event1=data1$event1
  event2=data1$event2
  event3=data1$event3
  
  res.BurrIII=MLE.GFGM.BurrIII(t.event,event1,event2,D,p,q,theta)
  
  Alpha.h = res.BurrIII$Alpha[1]
  Beta.h = res.BurrIII$Beta[1]
  Gamma.h = res.BurrIII$Gamma[1]
  
  res.spline = MLE.GFGM.spline(t.event,event1,event2,p,q,theta,h.plot=F)
  g1 = res.spline$g1
  g2 = res.spline$g2
  
  stat0=CvM.GFGM.BurrIII(t.event,event1,event2,Alpha.h,Beta.h,Gamma.h,g1,g2,p,q,theta,Sdist.plot =F)
  stat0=unlist(stat0)
  
  w.boot=max(t.event[event3==1])
  boot.sim=plyr::raply(B,gof.sim.BurrIII(n, Alpha.h, Beta.h, Gamma.h,w.boot),.progress=.progress)
  stat.boot=colMeans(boot.sim)
  q.boot=apply(boot.sim,2,quantile,probs=1-alpha)
  reject=stat0>q.boot
  res=c(stat0[1],stat.boot[1],reject[1],stat0[2],stat.boot[2],reject[2])
  names(res)=c("overall.stat","overall.boot.stat","overall.reject",
               "copula.stat","copula.boot.stat","copula.reject")
  return(res)
}

data.sim.Indep.exp <- function(n, Lambda1=0.5, Lambda2=1,w) {
  XY=Indep.exp(n, Lambda1, Lambda2)
  X=XY[,1]
  Y=XY[,2]
  cen=runif(n,0,w)
  XYC=cbind(XY,cen)
  
  t.event = apply(XYC, 1, min)
  event1  = as.integer(t.event==X)
  event2  = as.integer(t.event==Y)
  event3  = 1-event1-event2
  
  Time=apply(XYC, 1, which.min)
  Time=as.factor(Time)
  levels(Time) = c("Confirmed","Unconfirmed","Censoring")
  data1   = data.frame(t.event,Time,event1,event2,event3)
  return(data1)
}

boot.gof.Indep.exp <- function(n=200, Lambda1=0.5, Lambda2=1,w=6,
                            D=5000,p=3,q=2,theta=0.7,
                            id=NULL,B=500,alpha=0.05,.progress="text") {
  data1=data.sim.Indep.exp(n,  Lambda1, Lambda2,w)
  t.event=data1$t.event
  event1=data1$event1
  event2=data1$event2
  event3=data1$event3
  
  res.BurrIII=MLE.GFGM.BurrIII(t.event,event1,event2,D,p,q,theta)
  
  Alpha.h = res.BurrIII$Alpha[1]
  Beta.h = res.BurrIII$Beta[1]
  Gamma.h = res.BurrIII$Gamma[1]
  
  res.spline = MLE.GFGM.spline(t.event,event1,event2,p,q,theta,h.plot=F)
  g1 = res.spline$g1
  g2 = res.spline$g2
  
  stat0=CvM.GFGM.BurrIII(t.event,event1,event2,Alpha.h,Beta.h,Gamma.h,g1,g2,p,q,theta,Sdist.plot =F)
  stat0=unlist(stat0)
  
  w.boot=max(t.event[event3==1])
  boot.sim=plyr::raply(B,gof.sim.BurrIII(n, Alpha.h, Beta.h, Gamma.h,w.boot),.progress=.progress)
  stat.boot=colMeans(boot.sim)
  q.boot=apply(boot.sim,2,quantile,probs=1-alpha)
  reject=stat0>q.boot
  res=c(stat0[1],stat.boot[1],reject[1],stat0[2],stat.boot[2],reject[2])
  names(res)=c("overall.stat","overall.boot.stat","overall.reject",
               "copula.stat","copula.boot.stat","copula.reject")
  return(res)
}

# Log-Likelihood ------------------------------------------------

f1_func <- function(par) {
  Alpha <- exp(par[1])
  Beta <- exp(par[2])
  Gamma <- exp(par[3])
  H <- (1 + t.event^(-Gamma))^(-1)
  M1 <- Alpha * Gamma * t.event^(-Gamma - 1) * H^(Alpha +
                                                    1)
  M2 <- Alpha * Gamma * t.event^(-Gamma - 1) * H^(Alpha +
                                                    Beta + 1)
  K <- 0
  for (i in 0:q) {
    for (j in 0:(q - 1)) {
      K1 <- H^(Alpha * (p * j + 1) + Beta * (p * i +
                                               1) + 1)
      K <- K + choose(q, i) * choose(q - 1, j) * (-1)^(i +
                                                         j) * K1 * (1 - (1 + p * q) * H^(Alpha * p)) *
        t.event^(-Gamma - 1)
    }
  }
  return(M1 - M2 - theta * Alpha * Gamma * K)
}

f2_func <- function(par) {
  Alpha <- exp(par[1])
  Beta <- exp(par[2])
  Gamma <- exp(par[3])
  H <- (1 + t.event^(-Gamma))^(-1)
  M1 <- Beta * Gamma * t.event^(-Gamma - 1) * H^(Beta +
                                                   1)
  M2 <- Beta * Gamma * t.event^(-Gamma - 1) * H^(Alpha +
                                                   Beta + 1)
  K <- 0
  for (i in 0:q) {
    for (j in 0:(q - 1)) {
      K1 <- H^(Alpha * (p * i + 1) + Beta * (p * j +
                                               1) + 1)
      K <- K + choose(q, i) * choose(q - 1, j) * (-1)^(i +
                                                         j) * K1 * (1 - (1 + p * q) * H^(Beta * p)) *
        t.event^(-Gamma - 1)
    }
  }
  return(M1 - M2 - theta * Beta * Gamma * K)
}

Fb_func <- function(par) {
  Alpha <- exp(par[1])
  Beta <- exp(par[2])
  Gamma <- exp(par[3])
  H <- (1 + t.event^(-Gamma))^(-1)
  K <- 0
  for (i in 0:q) {
    for (j in 0:q) {
      K <- K + choose(q, i) * choose(q, j) * (-1)^(i +
                                                     j) * H^(Alpha * (p * j + 1) + Beta * (p *
                                                                                             i + 1))
    }
  }
  return(1 - H^Alpha - H^Beta + H^(Alpha + Beta) + theta *
           K)
}

logL_func <- function(par) {
  return(sum(event1 * log(f1_func(par))) + sum(event2 *
                                                 log(f2_func(par))) + sum((1 - event1 - event2) *
                                                                            log(Fb_func(par))))
}

# loglike <- function(par) {
#   t.event = t.event - eta
#   par=log(par)
#   return(logL_func(par))
# }
