library(flexsurv)
library(stats4)   # for mle
library(plyr)     # for raply
library(plot3D)   # for presp3D

#  joint PDF of T1, T2  -----------------------------------------

dT1T2=function(t1,t2,theta0, theta1, theta2, lambda){
  n=length(t1)
  tt1=t1
  tt2=t2
  d=rep(NA,n)
  for(i in 1:n){
    t1=tt1[i]
    t2=tt2[i]
    if(t1>t2){
      d[i]=dgompertz(t1,shape=lambda,rate=theta0+theta1)*
        dgompertz(t2,shape=lambda,rate=theta2)
    } else if(t1<t2){
      d[i]=dgompertz(t1,shape=lambda,rate=theta1)*
        dgompertz(t2,shape=lambda,rate=theta0+theta2)
    } else{
      d[i]=dgompertz(t1,shape=lambda,rate=theta1)*
        dgompertz(t2,shape=lambda,rate=theta0+theta2)
      # d[i]=theta0/(theta0+theta1+theta2)*
      #   dgompertz(t1,shape=lambda,rate=theta0+theta1+theta2)
    }
  }
  return(d)
}

# simulate ------------------------------------------------------

rPHCS <- function(n, theta0, theta1, theta2, lambda, m, r, tau) {
  y0=rgompertz(n, shape=lambda, rate =theta0)
  y1=rgompertz(n, shape=lambda, rate =theta1)
  y2=rgompertz(n, shape=lambda, rate =theta2)
  
  
  t1=pmin(y0,y1)
  t2=pmin(y0,y2)
  
  x0=pmin(t1,t2)
  alpha0=0*(t1==t2)+1*(t1<t2)+2*(t1>t2)
  alpha0=alpha0[order(x0)]
  x0=sort(x0)
  
  x=alpha=rep(NA,m)
  for(i in 1:m){
    x[i]=x0[1]
    alpha[i]=alpha0[1]
    n1=length(x0)
    id=sample(x = 2:n1,size = r[i])
    id=c(1,id)
    x0=x0[-id]
    alpha0=alpha0[-id]
  }
  
  tstar=ifelse(x[m]<tau,x[m],tau)
  
  x=x[x<tau]
  rr=length(x)
  alpha=alpha[1:rr]
  
  out=list(n=n,m=m,r=r,tstar=tstar,x=x,alpha=alpha)
  return(out)
}

# estimate -----------------------------------------------------

minuslogl <- function(theta0,theta1,theta2,lambda) {
  theta=c(theta0,theta1,theta2)
  nn=rep(NA,3)
  for(j in 0:2){
    nn[j+1]=sum(alpha==j)
  }
  names(nn)=0:2
  rr=sum(nn)
  c1=n-rr-sum(r)
  logl=sum(nn*theta)+lambda*sum(x)-
    (theta0+theta1+theta2)/lambda*
    (
      sum((r+1)*(exp(lambda*x)-1))+
      c1*(exp(lambda*tstar)-1)
    )
  return(-logl)
}

minuslogl2 <- function(pars) {
  theta0=pars[1]
  theta1=pars[2]
  theta2=pars[3]
  lambda=pars[4]
  theta=c(theta0,theta1,theta2)
  nn=rep(NA,3)
  for(j in 0:2){
    nn[j+1]=sum(alpha==j)
  }
  names(nn)=0:2
  rr=sum(nn)
  c1=n-rr-sum(r)
  logl=sum(nn*theta)+lambda*sum(x)-
    (theta0+theta1+theta2)/lambda*
    (
      sum((r+1)*(exp(lambda*x)-1))+
        c1*(exp(lambda*tstar)-1)
    )
  return(-logl)
}

glambda=function(lambda){
  nn=rep(NA,3)
  for(j in 0:2){
    nn[j+1]=sum(alpha==j)
  }
  names(nn)=0:2
  rr=sum(nn)
  c1=n-rr-sum(r)
  logl=rr*(
    log(lambda)-log(
      sum((r+1)*(exp(lambda*x))-1)+
        c1*(exp(lambda*tstar)-1)
      )
    )+ lambda*sum(x)
  return(-logl)
}

hlambda=function(lambda){
  nn=rep(NA,3)
  for(j in 0:2){
    nn[j+1]=sum(alpha==j)
  }
  names(nn)=0:2
  rr=sum(nn)
  c1=n-rr-sum(r)
  h=(sum((r+1)*x*exp(lambda*x))+c1*tstar*exp(lambda*tstar))/
    (sum((r+1)*(exp(lambda*x)-1))+c1*(exp(lambda*tstar)-1))-
    sum(x)/rr
  h=1/h
  return(h)
}

thetaLambda=function(lambda){
  nn=rep(NA,3)
  for(j in 0:2){
    nn[j+1]=sum(alpha==j)
  }
  names(nn)=0:2
  rr=sum(nn)
  c1=n-rr-sum(r)
  theta=nn*lambda/
    (
      sum((r+1)*(exp(lambda*x)-1))+
        c1*(exp(lambda*tstar)-1)
    )
  names(theta)=paste0("theta",names(nn))
  return(theta)
}

estimate <- function(lambda0, abstol=1e-4, maxit=1e4) {
  
  hlambda=function(lambda){
    nn=rep(NA,3)
    for(j in 0:2){
      nn[j+1]=sum(alpha==j)
    }
    names(nn)=0:2
    rr=sum(nn)
    c1=n-rr-sum(r)
    h=(sum((r+1)*x*exp(lambda*x))+c1*tstar*exp(lambda*tstar))/
      (sum((r+1)*(exp(lambda*x)-1))+c1*(exp(lambda*tstar)-1))-
      sum(x)/rr
    h=1/h
    return(h)
  }
  
  thetaLambda=function(lambda){
    nn=rep(NA,3)
    for(j in 0:2){
      nn[j+1]=sum(alpha==j)
    }
    names(nn)=0:2
    rr=sum(nn)
    c1=n-rr-sum(r)
    theta=nn*lambda/
      (
        sum((r+1)*(exp(lambda*x)-1))+
          c1*(exp(lambda*tstar)-1)
      )
    names(theta)=paste0("theta",names(nn))
    return(theta)
  }
  
  Lambda=lambda0
  i=1
  repeat {
    # cat(i, "")
    i=i+1
    Lambda[i]=hlambda(Lambda[i-1])
    d=abs(Lambda[i]-Lambda[i-1])
    if(d<abstol | i==maxit) {
      # cat(i, "\n")
      break
    }
  }
  lambdahat=Lambda[i]
  thetahat=thetaLambda(lambdahat)
  phat=c(thetahat,lambda=lambdahat)
  return(phat)
}

Ifunc=function(pars){
  theta0=pars[1]
  theta1=pars[2]
  theta2=pars[3]
  lambda=pars[4]
  theta=c(theta0,theta1,theta2)
  nn=rep(NA,3)
  for(j in 0:2){
    nn[j+1]=sum(alpha==j)
  }
  names(nn)=0:2
  rr=sum(nn)
  c1=n-rr-sum(r)
  
  s1=sum((r+1)*(exp(lambda*x)-1))+ c1*(exp(lambda*tstar)-1)
  s2=sum((r+1)*x*exp(lambda*x))+ c1*tstar*exp(lambda*tstar)
  s3=sum((r+1)*x^2*exp(lambda*x))+ c1*tstar^2*exp(lambda*tstar)
  
  Imat=matrix(0,4,4)
  diag(Imat)[1:3]=ifelse(nn==0,1,nn/theta^2)
  Imat[4,4]=(2*sum(theta)/lambda^3)*s1-
    (2*sum(theta)/lambda^2)*s2+
    (sum(theta)/lambda)*s3
  Imat[1:3,4]=Imat[4,1:3]=-(1/lambda^2)*s1+
    (1/lambda)*s2
  
  return(Imat)
}

Vfunc=function(Imat) solve(Imat)

ACIfunc=function(phat,v,gama=0.05){
  s=sqrt(diag(v))
  p=c(gama/2,1-gama/2)
  z=qnorm(p)
  lci=phat+z[1]*s
  uci=phat+z[2]*s
  lci=pmax(0,lci)
  ci=cbind(lci,uci)
  colnames(ci)=paste0("ACI_",p*100,"%")
  return(ci)
}

BootPCIfunc <- function(phat,n,m,r,tau,N=200,gama=0.05) {
  hlambda=function(lambda){
    nn=rep(NA,3)
    for(j in 0:2){
      nn[j+1]=sum(alpha==j)
    }
    names(nn)=0:2
    rr=sum(nn)
    c1=n-rr-sum(r)
    h=(sum((r+1)*x*exp(lambda*x))+c1*tstar*exp(lambda*tstar))/
      (sum((r+1)*(exp(lambda*x)-1))+c1*(exp(lambda*tstar)-1))-
      sum(x)/rr
    h=1/h
    return(h)
  }
  
  thetaLambda=function(lambda){
    nn=rep(NA,3)
    for(j in 0:2){
      nn[j+1]=sum(alpha==j)
    }
    names(nn)=0:2
    rr=sum(nn)
    c1=n-rr-sum(r)
    theta=nn*lambda/
      (
        sum((r+1)*(exp(lambda*x)-1))+
          c1*(exp(lambda*tstar)-1)
      )
    names(theta)=paste0("theta",names(nn))
    return(theta)
  }
  
  estimate <- function(lambda0, abstol=1e-4, maxit=1e4) {
    Lambda=lambda0
    i=1
    repeat {
      # cat(i, "")
      i=i+1
      Lambda[i]=hlambda(Lambda[i-1])
      d=abs(Lambda[i]-Lambda[i-1])
      if(d<abstol | i==maxit) {
        # cat(i, "\n")
        break
      }
    }
    lambdahat=Lambda[i]
    thetahat=thetaLambda(lambdahat)
    phat=c(thetahat,lambda=lambdahat)
    return(phat)
  }
  
  theta0=phat[1]
  theta1=phat[2]
  theta2=phat[3]
  lambda=phat[4]
  pmat=matrix(NA,N,4)
  colnames(pmat)=names(phat)
  for(i in 1:N){
    res=rPHCS(n,theta0,theta1,theta2,lambda,m,r,tau)
    xstar=res$x
    alpha=res$alpha
    tstar=res$tstar

    w=runif(m)
    RR=rev(r)
    RRR=cumsum(RR)
    j=1:m
    v=w^(1/(j+RRR))
    VV=rev(v)
    VVV=cumprod(VV)
    u=1-VVV
    
    eq1=1-exp(-(theta0+theta1)/lambda*(exp(lambda*tau)-1))
    eq2=1-exp(-(theta0+theta2)/lambda*(exp(lambda*tau)-1))
    
    if(u[m]<= eq1){
      rr=m;
      tstar=log(1-lambda/(theta0+theta1)*log(1-u[m]))/lambda
    } else if(u[m]<= eq2){
      rr=m;
      tstar=log(1-lambda/(theta0+theta2)*log(1-u[m]))/lambda
    } else if(u[m]> eq1){
      rr=J=max(which(u<eq1))
      tstar=tau
      xstar=log(1-lambda/(theta0+theta1)*log(1-u[1:rr]))/lambda
    } else if(u[m]> eq2){
      rr=J=max(which(u<eq2))
      tstar=tau
      xstar=log(1-lambda/(theta0+theta2)*log(1-u[1:rr]))/lambda
    }
    
    x=xstar
    pmat[i,]=estimate(lambda0 = lambda)
  }
  ci=apply(pmat,2,quantile,probs=c(gama/2,1-gama/2))
  ci=t(ci)
  colnames(ci)=paste0("BootPCI_",colnames(ci))
  return(ci)
  # return(pmat)
}

cover.test=function(CI,pars){
  test=(CI[,1]<=pars) & (pars<=CI[,2])
  return(test)
}



simPHCS <- function(pars, n, m, r, tau=1,N=200,gama=0.05,maxk=30) {
  
  estimate <- function(lambda0, abstol=1e-4, maxit=1e4) {
    
    hlambda=function(lambda){
      nn=rep(NA,3)
      for(j in 0:2){
        nn[j+1]=sum(alpha==j)
      }
      names(nn)=0:2
      rr=sum(nn)
      c1=n-rr-sum(r)
      h=(sum((r+1)*x*exp(lambda*x))+c1*tstar*exp(lambda*tstar))/
        (sum((r+1)*(exp(lambda*x)-1))+c1*(exp(lambda*tstar)-1))-
        sum(x)/rr
      h=1/h
      return(h)
    }
    
    thetaLambda=function(lambda){
      nn=rep(NA,3)
      for(j in 0:2){
        nn[j+1]=sum(alpha==j)
      }
      names(nn)=0:2
      rr=sum(nn)
      c1=n-rr-sum(r)
      theta=nn*lambda/
        (
          sum((r+1)*(exp(lambda*x)-1))+
            c1*(exp(lambda*tstar)-1)
        )
      names(theta)=paste0("theta",names(nn))
      return(theta)
    }
    
    Lambda=lambda0
    i=1
    repeat {
      # cat(i, "")
      i=i+1
      Lambda[i]=hlambda(Lambda[i-1])
      d=abs(Lambda[i]-Lambda[i-1])
      if(d<abstol | i==maxit) {
        # cat(i, "\n")
        break
      }
    }
    lambdahat=Lambda[i]
    thetahat=thetaLambda(lambdahat)
    phat=c(thetahat,lambda=lambdahat)
    return(phat)
  }
  
  Ifunc=function(pars){
    theta0=pars[1]
    theta1=pars[2]
    theta2=pars[3]
    lambda=pars[4]
    theta=c(theta0,theta1,theta2)
    nn=rep(NA,3)
    for(j in 0:2){
      nn[j+1]=sum(alpha==j)
    }
    names(nn)=0:2
    rr=sum(nn)
    c1=n-rr-sum(r)
    
    s1=sum((r+1)*(exp(lambda*x)-1))+ c1*(exp(lambda*tstar)-1)
    s2=sum((r+1)*x*exp(lambda*x))+ c1*tstar*exp(lambda*tstar)
    s3=sum((r+1)*x^2*exp(lambda*x))+ c1*tstar^2*exp(lambda*tstar)
    
    Imat=matrix(0,4,4)
    diag(Imat)[1:3]=ifelse(nn==0,1,nn/theta^2)
    Imat[4,4]=(2*sum(theta)/lambda^3)*s1-
      (2*sum(theta)/lambda^2)*s2+
      (sum(theta)/lambda)*s3
    Imat[1:3,4]=Imat[4,1:3]=-(1/lambda^2)*s1+
      (1/lambda)*s2
    
    return(Imat)
  }
  
  Vfunc=function(Imat) solve(Imat)
  
  ACIfunc=function(phat,v,gama=0.05){
    s=sqrt(diag(v))
    p=c(gama/2,1-gama/2)
    z=qnorm(p)
    lci=phat+z[1]*s
    uci=phat+z[2]*s
    lci=pmax(0,lci)
    ci=cbind(lci,uci)
    colnames(ci)=paste0("ACI_",p*100,"%")
    return(ci)
  }
  
  theta0=pars[1]
  theta1=pars[2]
  theta2=pars[3]
  lambda=pars[4]
  ACI=NA; k=0
  while(anyNA(ACI) & k<maxk){
    k=k+1
    res=rPHCS(n,theta0,theta1,theta2,lambda,m,r,tau)
    x=res$x
    alpha=res$alpha
    tstar=res$tstar
    phat=estimate(lambda0 = lambda)
    pars=c(theta0=theta0,theta1=theta1,theta2=theta2,lambda=lambda)
    Ihat=Ifunc(phat)
    Vhat=Vfunc(Ihat)
    ACIhat=ACIfunc(phat,Vhat,gama)
    ACI=cover.test(ACIhat,pars)
  }
  
  BootPCIhat=BootPCIfunc(phat,n,m,r,tau,N,gama)
  BootP=cover.test(BootPCIhat,pars)
  out=cbind(phat=phat,ACI=ACI,BootP=BootP)
  return(out)
}
